package be.iminds.ilabt.jfed.experimenter_gui.dialogs;

import be.iminds.ilabt.jfed.experiment.Experiment;
import be.iminds.ilabt.jfed.experiment.ExperimentController;
import be.iminds.ilabt.jfed.experiment.ExperimentPart;
import be.iminds.ilabt.jfed.experimenter_gui.slice.jobs.TestLinksJob;
import be.iminds.ilabt.jfed.experimenter_gui.slice.scp.SCPTask;
import be.iminds.ilabt.jfed.experimenter_gui.util.DateTimeUtils;
import be.iminds.ilabt.jfed.highlevel.model.InternalState;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.ui.javafx.FXMLUtil;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import be.iminds.ilabt.jfed.ui.javafx.util.InformationLabel;
import javafx.beans.binding.Bindings;
import javafx.beans.binding.DoubleBinding;
import javafx.beans.property.*;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.concurrent.Worker;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.control.*;
import javafx.scene.control.cell.CheckBoxTableCell;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.stage.Window;
import javafx.util.Pair;
import org.controlsfx.glyphfont.FontAwesome;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * User: twalcari
 * Date: 2/24/14
 * Time: 4:03 PM
 */

public class LinkTestDialog extends VBox {
    @Nullable private final Window parentWindow;

    private final Dialog<Void> dialog;
    @Nonnull private final ObservableList<TestLinksJob.LinkTestReport> reports =
            FXCollections.observableArrayList();


    @FXML
    private TableView<TestLinksJob.LinkTestReport> linkTestTableView;

    @FXML
    private TableColumn<TestLinksJob.LinkTestReport, String> nodeTableColumn;
    @FXML
    private TableColumn<TestLinksJob.LinkTestReport, String> linkToTableColumn;
    @FXML
    private TableColumn<TestLinksJob.LinkTestReport, Pair<Boolean, String>> connectivityTableColumn;
    @FXML
    private TableColumn<TestLinksJob.LinkTestReport, Pair<Boolean, String>> speedHeaderColumn;
    @FXML
    private TableColumn<TestLinksJob.LinkTestReport, Pair<Boolean, String>> bwTableColumn;
    @FXML
    private TableColumn<TestLinksJob.LinkTestReport, Pair<Boolean, String>> linkSpeedTableColumn;
    @FXML
    private TableColumn<TestLinksJob.LinkTestReport, Number> expectedSpeedColumn;

    @FXML
    private InformationLabel sliceTerminationLabel;


    LinkTestDialog(@Nullable Window parentWindow,
                   @Nonnull List<TestLinksJob.LinkTestReport> reports) {
        this.parentWindow = parentWindow;
        this.reports.addAll(reports);

        FXMLUtil.injectFXML(this);

        DialogPane dialogPane = new DialogPane();
        dialogPane.setHeaderText("Link Test Results");
        dialogPane.setGraphic(GlyphUtils.createDialogGlyph(FontAwesome.Glyph.STETHOSCOPE, Color.BLUE));
        dialogPane.setContent(this);
        dialogPane.getButtonTypes().setAll(ButtonType.CLOSE);

        dialog = new Dialog<>();
        if (parentWindow != null)
            dialog.initOwner(parentWindow);
        dialog.setDialogPane(dialogPane);
        dialog.setTitle("Link Test Results");
    }

    @FXML
    private void initialize() {
        //populate table

        double lastColumnWidth = connectivityTableColumn.getWidth() +
                speedHeaderColumn.getWidth();
//                expectedSpeedColumn.getWidth() +
//                bwTableColumn.getWidth();
//                linkSpeedTableColumn.getWidth();
        DoubleBinding dividableWidth =
                linkTestTableView.widthProperty().subtract(lastColumnWidth).subtract(5);
        nodeTableColumn.prefWidthProperty().bind(dividableWidth.multiply(0.5));
        linkToTableColumn.prefWidthProperty().bind(dividableWidth.multiply(0.5));

        linkTestTableView.setItems(reports);
        linkTestTableView.setPrefHeight(26/*header*/ + reports.size() * 25);

        nodeTableColumn.setCellValueFactory(t -> t.getValue().getFrom().clientIdProperty());
        linkToTableColumn.setCellValueFactory(t -> t.getValue().getTo().clientIdProperty());
        connectivityTableColumn.setCellValueFactory(t ->
                new SimpleObjectProperty<>(new Pair<>(t.getValue().isConnectivityOk(), null)));
        bwTableColumn.setCellValueFactory(t ->
                new SimpleObjectProperty<>(new Pair<>(t.getValue().isThroughputOk(),
                        t.getValue().getActualThroughput_mbps()+"")));
        expectedSpeedColumn.setCellValueFactory(t ->
                new SimpleIntegerProperty(t.getValue().getExpectedLinkSpeed_mbps()));
        linkSpeedTableColumn.setCellValueFactory(t -> {
            return new SimpleObjectProperty<>(
                    new Pair<>(t.getValue().isLinkSpeedSettingOk(),
                            t.getValue().getActualLinkSpeedSetting_mbps()+"")
            );
        });

        connectivityTableColumn.setCellFactory((col) -> new SummaryTableCell());
        bwTableColumn.setCellFactory((col) -> new SummaryTableCell());
        linkSpeedTableColumn.setCellFactory((col) -> new SummaryTableCell());
    }


    public void showDialog() {
        dialog.showAndWait();
    }


    private static class SummaryTableCell extends TableCell<TestLinksJob.LinkTestReport, Pair<Boolean, String>> {
        private final Label summaryLabel = new Label();

        @Override
        public void updateItem(Pair<Boolean, String> item, boolean empty) {
            super.updateItem(item, empty);

            if (empty) {
                setGraphic(null);
            } else {
                summaryLabel.setGraphic(
                        item.getKey() ?
                                GlyphUtils.createSmallGlyph(FontAwesome.Glyph.THUMBS_UP, Color.GREEN) :
                                GlyphUtils.createSmallGlyph(FontAwesome.Glyph.THUMBS_DOWN, Color.RED)
                );
                summaryLabel.setText(item.getValue());
                setGraphic(summaryLabel);
            }
        }
    }
}
